/* ============================= C MeatAxe ==================================
   File:        $Id: zpo.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Power of a matrix or permutation.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

#include <ctype.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>




/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

MTX_DEFINE_FILE_INFO

static int Power;

static MtxApplicationInfo_t AppInfo = { 
"power", "Power of a matrix or permutation", 
"SYNTAX\n"
"    zpo [<Options>] <In> <n> <Result>\n"
"\n"
"OPTIONS\n"
MTX_COMMON_OPTIONS_DESCRIPTION
"\n"
"ARGUMENTS\n"
"    <In> ........... Input: Matrix or permutation\n"
"    <n> ............ Power to compute\n"
"    <Result> ....... Ouput: <n>-th power of <In>\n"
};

static MtxApplication_t *App = NULL;








/* ------------------------------------------------------------------
   init() - Process command line options and arguments
   ------------------------------------------------------------------ */

static int Init(int argc, const char **argv)

{
    /* Process command line options.
       ----------------------------- */
    App = AppAlloc(&AppInfo,argc,argv);
    if (App == NULL)
	return -1;

    /* Process arguments.
       ------------------ */
    if (AppGetArguments(App,3,3) < 0)
	return -1;
    Power = atoi(App->ArgV[1]);

    return 0;
}




static void Cleanup()

{
    AppFree(App);
}



static int CalcPower()

{
    void *x = XLoad(App->ArgV[0]);
    void *y;

    if (x == NULL)
	return -1;
    y = XPower(x,Power);
    if (y == NULL)
	return -1;
    XSave(y,App->ArgV[2]);
    XFree(x);
    XFree(y);
    return 0;

}






/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, const char **argv)

{
    if (Init(argc,argv) != 0)
    {
	MTX_ERROR("Initialization failed");
	return 1;
    }

    CalcPower();

    Cleanup();
    return EXIT_OK;
}




/*{{{ --------------------------------------------------------------------------
!section apps.general
!program zpo "Power"
!synopsis 
    zpo [<Options>] <In> <n> <Result>
!arg n
    Power.
!arg In
    Matrix or permutation.
!arg Result
    Output: n-th power of the matrix or permutation.
!description
    This program calculates the n-th power of a matrix or permutation.
    {\em n} may be negative.
!seealso 
-------------------------------------------------------------------------- }}}*/


